/*
 // Copyright (c) 2017 Intel Corporation
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
 //
 //      http://www.apache.org/licenses/LICENSE-2.0
 //
 // Unless required by applicable law or agreed to in writing, software
 // distributed under the License is distributed on an "AS IS" BASIS,
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
 */
 
#pragma once

#include <string>

#include <inference_engine.hpp>
#include <ie_plugin_ptr.hpp>
#include <mkldnn/mkldnn_extension_ptr.hpp>

using MKLDNNExtensionPtr = std::shared_ptr<InferenceEngine::MKLDNNPlugin::IMKLDNNExtension>;

class Network;

using NetworkPtr = std::shared_ptr<Network>;
using NetworkWeakPtr = std::weak_ptr<Network>;

using NetworkDataMap = std::map<std::string, InferenceEngine::Blob::Ptr>;

class Network {
public:
    Network(InferenceEngine::InferenceEnginePluginPtr plugin);
    ~Network();

public:
    void LoadNetwork(const std::string& pathXml, const std::string& pathBin, size_t batch);
    void LoadNetwork(const std::string& pathXml, size_t batch);

    void AddExtensionToPlugin(MKLDNNExtensionPtr& extension) {
        _plugin->AddExtension(std::dynamic_pointer_cast<InferenceEngine::IExtension>(extension), nullptr);
    }

    void Infer();

    InferenceEngine::BlobMap& GetInputMap() {
        return _inputMap;
    }

    InferenceEngine::BlobMap& GetOutputMap() {
        return _outputMap;
    }

    std::vector<size_t> GetInputDims(const std::string& layerName = "");
    std::vector<size_t> GetOutputDims(const std::string& layerName = "");

    size_t GetInputSize(const std::string& layerName = "") {
        return AccumDims(GetInputDims(layerName));
    }

    size_t GetOutputSize(const std::string& layerName = "") {
        return AccumDims(GetOutputDims(layerName));
    }

    void AddOutput(const std::string& layerName);

    void LoadNetworkToPlugin();

protected:
    bool IsNetworkLoaded() const { return (_network != nullptr && _network->isParseSuccess()); }

    void InitInputs();
    void InitOutputs();

    size_t AccumDims(const std::vector<size_t>& dims);
    InferenceEngine::Blob::Ptr MakeSharedBlob(InferenceEngine::Precision precision, InferenceEngine::SizeVector dims);

private:
    InferenceEngine::CNNNetReader::Ptr _network;
    InferenceEngine::InferenceEnginePluginPtr _plugin;
    bool _networkLoadedToPlugin;

    InferenceEngine::BlobMap _inputMap;
    InferenceEngine::BlobMap _outputMap;
};
