/*
// Copyright (c) 2017 Intel Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
*/

#pragma once

#include <string>
#include <vector>

#include <ie_plugin_dispatcher.hpp>
#include <ie_plugin_ptr.hpp>

std::string GetArchPath();

InferenceEngine::InferenceEnginePluginPtr SelectPlugin(const std::vector<std::string> &pluginDirs,
                                            const std::string &plugin, InferenceEngine::TargetDevice device);

InferenceEngine::InferenceEnginePluginPtr SelectPlugin(const std::vector<std::string> &pluginDirs,
                                                       const std::string &plugin, const std::string& device);

std::string FileNameNoPath(const std::string& filePath);
std::string FileNameNoExt(const std::string& filePath);
std::string FilePath(const std::string& filePath);
std::string FileExt(const std::string& filePath);

char FilePathSeparator();

void GetFilesInDir(const std::string& path, const std::vector<std::string>& exts, std::vector<std::string>& files);

bool IsDirectory(const std::string& path);

std::string ToUpper(const std::string& str);
std::string& Trim(std::string &s);
bool ParseFloats(const std::string& line, std::vector<float>& result, const std::vector<float>& def);

std::string GetOSSpecificLibraryName(const std::string& lib);

void ReadLabels(const char *path, std::vector<std::string> &labels);

void ConvertBGR(const size_t imageWidth, const size_t imageHeight, const std::vector<uint8_t*>& srcs, float* dst,
                const std::vector<float>& mean, const std::vector<float>& scale);
void ConvertGRAY(const size_t imageWidth, const size_t imageHeight, const std::vector<uint8_t*>& srcs, float* dst);

